package ai.pai.lensman.dslr;


import android.app.Service;
import android.content.Intent;
import android.graphics.Bitmap;
import android.os.AsyncTask;
import android.os.Bundle;
import android.os.Handler;
import android.os.IBinder;
import android.os.Message;
import android.os.Process;
import android.text.TextUtils;

import com.android.common.utils.LogHelper;
import com.remoteyourcam.usb.ptp.Camera;
import com.remoteyourcam.usb.ptp.PtpConstants;
import com.remoteyourcam.usb.ptp.PtpService;
import com.remoteyourcam.usb.ptp.model.LiveViewData;
import com.remoteyourcam.usb.ptp.model.ObjectInfo;

import java.io.File;
import java.io.FileOutputStream;
import java.util.ArrayList;

import ai.pai.lensman.db.Preferences;

public class CameraService extends Service implements Handler.Callback, Camera.CameraListener, Camera.StorageInfoListener, Camera.RetrieveImageInfoListener {

    private Handler mainHandler;
    private final static int MSG_CHECK_STORAGE = 100;

    public static final String ACTION_CAMERA_SERVICE_STATUS_CHANGE = "action.ai.pai.lensman.dslr.cameraservice";
    public static final String EXTRA_STATUS_PART = "status";
    public static final String EXTRA_DATA_PART = "data";

    public static final int MSG_CAMERA_INIT_SUCCESS = 8000;
    public static final int MSG_CAMERA_CONN_ERROR = 8001;
    public static final int MSG_CAMERA_NEW_PHOTO_FOUND = 8002;
    public static final int MSG_SDCARD_ERROR = 8003;


    public static final String EXTRA_CMD = "cmd";
    public static final String EXTRA_SESSION_DIR = "session_dir";
    public static final int CMD_INIT_CAMERA_CONNECTION = 9000;
    public static final int CMD_EXIT_CAMERA_CONNECTION = 9001;
    public static final int CMD_START_CAPTURE_PHOTO = 9002;
    public static final int CMD_STOP_CAPTURE_PHOTO = 9003;

    private PtpService ptp;
    private Camera camera;
    private int[] origin = new int[0];
    private String sessionDir;

    @Override
    public void onCreate() {
        super.onCreate();
        ptp = PtpService.Singleton.getInstance(this);
        mainHandler = new Handler(this);
    }

    @Override
    public IBinder onBind(Intent intent) {
        return null;
    }

    @Override
    public int onStartCommand(Intent intent, int flags, int startId) {
        LogHelper.d("czy", "CameraService onStartCommand  ");
        if (camera == null) {
            startCamera(intent);
        } else if (intent != null && intent.getIntExtra(EXTRA_CMD, 0) > 0) {
            int cmd = intent.getIntExtra(EXTRA_CMD, 0);
            if (cmd == CMD_EXIT_CAMERA_CONNECTION) {
                LogHelper.d("czy", "CameraService 收到停止进程任务");
                stopCamera();
            } else if (cmd == CMD_INIT_CAMERA_CONNECTION) {
                if (camera != null) {
                    Bundle bundle = new Bundle();
                    bundle.putInt(EXTRA_STATUS_PART, MSG_CAMERA_INIT_SUCCESS);
                    sendCameraIntent(bundle);
                }
            } else if (cmd == CMD_START_CAPTURE_PHOTO) {
                if (camera != null) {
                    sessionDir = intent.getStringExtra(EXTRA_SESSION_DIR);
                    if (!TextUtils.isEmpty(sessionDir)) {
                        startCapture();
                        LogHelper.d("czy", "CameraService 收到开始拍摄任务");
                    }

                }
            } else if (cmd == CMD_STOP_CAPTURE_PHOTO) {
                LogHelper.d("czy", "CameraService 收到结束拍摄任务");
                stopCapture();
            }
        }
        return super.onStartCommand(intent, flags, startId);
    }

    @Override
    public void onDestroy() {
        stopCamera();
        super.onDestroy();
    }

    private void startCamera(Intent intent) {
        LogHelper.d("czy", "CameraService 尝试与相机建立连接");
        ptp.setCameraListener(this);
        ptp.initialize(this, intent);
    }

    private void stopCamera() {
        LogHelper.d("czy", "CameraService stopCameraService  ");
        mainHandler.removeCallbacksAndMessages(null);
        ptp.setCameraListener(null);
        ptp.shutdown();
        stopSelf();
        Process.killProcess(Process.myPid());
        System.exit(0);
    }


    public void startCapture() {
        stopCapture();
        mainHandler.sendEmptyMessage(MSG_CHECK_STORAGE);
    }

    public void stopCapture() {
        mainHandler.removeCallbacksAndMessages(null);
        origin = new int[0];
    }

    @Override
    public boolean handleMessage(Message msg) {
        if (msg.what == MSG_CHECK_STORAGE) {
            if (camera != null) {
                mainHandler.removeMessages(MSG_CHECK_STORAGE);
                camera.retrieveImageHandles(CameraService.this, 0xFFFFFFFF, PtpConstants.ObjectFormat.EXIF_JPEG);
                mainHandler.sendEmptyMessageDelayed(MSG_CHECK_STORAGE, Preferences.getInstance().getCameraQueryInterval());
            }
        }
        return false;
    }

    private void sendCameraIntent(Bundle bundle) {
        Intent intent = new Intent(ACTION_CAMERA_SERVICE_STATUS_CHANGE);
        intent.putExtras(bundle);
        sendBroadcast(intent);
    }


    @Override
    public void onCameraStarted(Camera camera) {
        this.camera = camera;
        LogHelper.d("czy", "CameraService 相机初始化成功");
        Bundle bundle = new Bundle();
        bundle.putInt(EXTRA_STATUS_PART, MSG_CAMERA_INIT_SUCCESS);
        sendCameraIntent(bundle);
    }

    @Override
    public void onCameraStopped(Camera camera) {
        this.camera = null;
        Bundle bundle = new Bundle();
        bundle.putInt(EXTRA_STATUS_PART, MSG_CAMERA_CONN_ERROR);
        sendCameraIntent(bundle);
    }

    @Override
    public void onNoCameraFound() {
        this.camera = null;
        Bundle bundle = new Bundle();
        bundle.putInt(EXTRA_STATUS_PART, MSG_CAMERA_CONN_ERROR);
        sendCameraIntent(bundle);
    }

    @Override
    public void onError(String message) {
        camera = null;
        Bundle bundle = new Bundle();
        bundle.putInt(EXTRA_STATUS_PART, MSG_CAMERA_CONN_ERROR);
        sendCameraIntent(bundle);
    }

    @Override
    public void onPropertyChanged(int property, int value) {

    }

    @Override
    public void onPropertyStateChanged(int property, boolean enabled) {

    }

    @Override
    public void onPropertyDescChanged(int property, int[] values) {

    }

    @Override
    public void onLiveViewStarted() {

    }

    @Override
    public void onLiveViewData(LiveViewData data) {

    }

    @Override
    public void onLiveViewStopped() {

    }

    @Override
    public void onCapturedPictureReceived(int objectHandle, String filename, Bitmap thumbnail, Bitmap bitmap) {

        LogHelper.d("czy", "CameraService fetchPhotoTask new photo found");
        saveBmpAndNotify(objectHandle, filename, bitmap);
    }

    @Override
    public void onBulbStarted() {

    }

    @Override
    public void onBulbExposureTime(int seconds) {

    }

    @Override
    public void onBulbStopped() {

    }

    @Override
    public void onFocusStarted() {

    }

    @Override
    public void onFocusEnded(boolean hasFocused) {

    }

    @Override
    public void onFocusPointsChanged() {

    }

    @Override
    public void onObjectAdded(int handle) {
        LogHelper.d("czy", "CameraService onObjectAdded: " + handle);
    }

    @Override
    public void onLogMessage(String msg) {

    }

    @Override
    public void onStorageFound(int handle, String label) {

    }

    @Override
    public void onAllStoragesFound() {

    }

    @Override
    public void onImageHandlesRetrieved(int[] handles) {
        LogHelper.d("czy", "CameraService onImageHandlesRetrieved: " + handles.length);

        if (origin.length == 0) {
            origin = handles;
        } else {
            if (origin.length == handles.length) {
                return;
            }
            ArrayList<Integer> diff = new ArrayList<>();
            for (int i = 0; i < handles.length; i++) {
                boolean isMatch = false;
                for (int j = 0; j < origin.length; j++) {
                    if (origin[j] == handles[i]) {
                        isMatch = true;
                        break;
                    }
                }
                if (!isMatch) {
                    diff.add(handles[i]);
                }
            }
            if (camera != null && diff.size() > 0) {
                for (int k = 0; k < diff.size(); k++) {
                    LogHelper.d("czy", "CameraService fetchPhotoTask new photo found: " + diff.get(k));
//                    camera.retrievePicture(diff.get(k));
                }
            }
            origin = handles;
        }
    }

    @Override
    public void onImageInfoRetrieved(int objectHandle, ObjectInfo objectInfo, Bitmap thumbnail) {

    }

    private void saveBmpAndNotify(int objectHandle, String filename, Bitmap bmp) {

        new SaveBitmapTask(objectHandle, filename, bmp).execute();
    }

    class SaveBitmapTask extends AsyncTask<Void, Integer, Integer> {
        private String filename;
        private Bitmap bitmap;
        private int handle;

        SaveBitmapTask(int objectHandle, String filename, Bitmap bmp) {
            this.filename = filename;
            this.bitmap = bmp;
            this.handle = objectHandle;
        }

        private boolean isCancelled = false;

        public void cancel() {
            isCancelled = true;
        }

        @Override
        protected Integer doInBackground(Void... params) {
            try {
                File dir = new File(sessionDir);
                if(!dir.exists()){
                    dir.mkdir();
                }
                File file = new File(dir,handle +".jpg");
                FileOutputStream fos = new FileOutputStream(file);
                bitmap.compress(Bitmap.CompressFormat.JPEG,100,fos);
                fos.flush();
                fos.close();
                return 0;
            } catch (Throwable t) {
                t.printStackTrace();
                return -1;
            }
        }

        @Override
        protected void onPostExecute(Integer result) {
            if (isCancelled) {
                return;
            }
            if (result >= 0) {
                Bundle bundle = new Bundle();
                bundle.putInt(EXTRA_STATUS_PART, MSG_CAMERA_NEW_PHOTO_FOUND);
                bundle.putString(EXTRA_DATA_PART, filename);
                sendCameraIntent(bundle);
            }
        }
    }

}
